<?php
/**
 * Process the customer payment using the braintree PHP SDK. The $order_id is used to fetch the order amount to be charged.
 * @param int $order_id
 */
function process_customer_payment($order_id, Braintree_For_Woocommerce $gateway){
	global $woocommerce;
	
	$order = new WC_Order($order_id);
	if(!$order) {
		wc_add_notice(__('There has been an issue processing your order. Please try again', 'woocommerce'), 'error');
		return;
	}
	if(WC_Subscriptions_Order::is_item_subscription($order)){
		return apply_filters('process_braintree_subscription', $order, $gateway);
	}
	
	return apply_filters('process_braintree_payment', $order, $gateway);
		
}
add_filter('process_customer_payment', 'process_customer_payment', 10, 2);
		




/**
 * Trigger the Braintree payment processing using the payment nonce. 
 * @param WC_Order $order_array
 */
function process_braintree_payment(WC_Order $order, Braintree_For_Woocommerce $gateway){
	
	$amount = $order->get_total();
	
	$nonce = ($gateway->environment == 'sandbox') ? 'fake-valid-nonce' : $_REQUEST['payment_method_nonce'];
	$save_payment = 'save' == $_POST['save_cc'] ? true : false;
	$paypal = 'PayPalAccount' == $_POST['payment_type'] ? true : false;
	$creditCard = 'CreditCard' == $_POST['payment_type'] ? true : false;
	$braintree_id = null;
	
	if($user_id = wp_get_current_user()->ID){
		$braintree_id = get_user_meta($user_id, 'braintree_vault_id', true);
	}

	
	
	$user_id = apply_filters('get_user_for_payment', $user_id);
	
	$args = array(
			'amount'=>$amount,
			'orderId'=>$order->id,
			'options'=>array(
					'submitForSettlement'=>true
			),
			'customer'=>array(
					'firstName'=>$_POST['billing_first_name'],
					'lastName'=>$_POST['billing_last_name']
			)
		
	);
	
	if(!empty($_POST['use_saved_card'])){
		$token = $_POST['existing_card'];
	}
	if(!isset($_REQUEST['payment_method_nonce']) && !$token){
		wc_add_notice(__('There has been an error verifying your payment', 'woocommerce'), 'error');
		return;
	}
	
		if($paypal || $creditCard && !$token){
			$args['paymentMethodNonce'] = $nonce;
	  	
	}
	if($token){
		$args['paymentMethodToken'] = $token;
		$save_payment = false;
		$args['customerId'] = $braintree_id;
	}
	
	$args = apply_filters('braintree_payment_arguments', $args);
	
	if($save_payment && $user_id){
		$args['options']['storeInVaultOnSuccess'] = true;
		
		if($braintree_id){
			if(has_braintree_id($braintree_id)){
				
				$result = Braintree_PaymentMethod::create(array(
						'customerId'=>$braintree_id,
						'paymentMethodNonce'=>$nonce
				));
				if($result->success){
				$args['paymentMethodToken'] = $result->paymentMethod->token;
				unset($args['paymentMethodNonce']);
				}
			}
		}
			else {
				$args['paymentMethodNonce'] = $nonce;
			}
		}
		
	$result = Braintree_Transaction::sale($args);
	
	do_action('process_braintree_transaction_result', $result, $order);
	
	if($result->success){
		global $woocommerce;
		if($gateway->order_status == 'wc-completed'){
			$order->payment_complete($result->transaction->id);
		}
		else{
			$order->update_status($gateway->order_status, __('Credit card payment processed', 'woocommerce'));
		}
		$woocommerce->cart->empty_cart();
			if($user_id && !$braintree_id){
				$new_id = $result->transaction->customerDetails->id;
				update_user_meta($user_id, 'braintree_vault_id', $new_id);
			}
		return array(
				'result'=>'success',
				'redirect'=>WC_Payment_Gateway::get_return_url($order));
	}
	elseif($result->transaction){
		wc_add_notice($result->message, 'error');
		return array(
				'result'=>'failure',
				'redirect'=>''
		);
	}
	else {
		$message = $result->_attributes->message;
		foreach($result->errors->deepAll() as $error){
			wc_add_notice($error->message, 'error');
		}
		return array(
				'result'=>'fail',
				'redirect'=>''
		);
	}
	
}
add_filter('process_braintree_payment', 'process_braintree_payment', 10, 2);


function get_credit_cardTypes(){
	$card_types = array('American Express', 'MasterCard', 'Discover', 'Visa', 'Diners Club', 'Carte', 'China UnionPay',  
			'JCB', 'Laser', 'Maestro');
	return $card_types;
}

function card_type_html($card_types){
	foreach($card_types as $card){
		$html .= '<option id="'.$card.'" class="cc_option">'.$card.'</option>';
	}
	return $html;
}

function get_expiration_months_html(){
	/* $months = array(
			'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September',
			'October', 'November', 'December'); */
	$months = array('01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12');
	
	//$count = 1;
	foreach($months as $month){
		$html .= '<option id="'.$month.'">'.$month.'</option>';
	}
	return $html;
}

function get_expiration_years(){
	$current_year = (int) date('Y');
	$count = 10;
	for($i = 0; $i < $count; $i++){
		$html .= '<option class="expire_year">'.$current_year.'</option>';
		$current_year++;
	}
	return $html;
}

function send_braintree_client_token($user = ''){
	if($user){
	$user_id = $user->id;
	$html = '<input type="hidden" id="client_token" value="'.Braintree_ClientToken::generate(array('customerId'=>$user_id)).'"/>';
	return $html;
	 }
	 try{
	 	$client_token = Braintree_ClientToken::generate();
	 }
	 catch(Exception $e){
	 	$client_token = '';
	 }
	$html = '<input type="hidden" id="client_token" value="'.$client_token.'"/>';
	return $html;
}

function get_braintree_checkout_ui(Braintree_For_Woocommerce $gateway){
	if($gateway->form_type == 'custom'){
		get_custom_checkout_ui($gateway);
	}
	else {
		get_dropin_ui($gateway);
	}
}
add_filter('get_braintree_checkout_ui', 'get_braintree_checkout_ui', 10, 2);

function get_custom_checkout_ui(Braintree_For_Woocommerce $gateway){
	$wp_user = wp_get_current_user();
	if($wp_user){
		$braintree_id = get_user_meta($wp_user->ID, 'braintree_vault_id', true);
	}
	$user = get_current_braintree_user($braintree_id);
	$user = apply_filters('get_braintree_user', $user, $gateway);
	
	switch ($gateway->payment_form_name){
		case 'payment_form_0':
			return call_user_func('braintree_payment_form_0', array('gateway'=>$gateway, 'user'=>$user));
			break;
			
		case 'payment_form_1':
			return call_user_func('braintree_payment_form_1', array('gateway'=>$gateway, 'user'=>$user));
			break;	
			
		case 'payment_form_2':
			return call_user_func('braintree_payment_form_2', array('gateway'=>$gateway, 'user'=>$user));
			break;
			
	}
	
}

function get_dropin_ui($gateway){
	$wp_user = wp_get_current_user();
	if($wp_user){
		$braintree_id = get_user_meta($wp_user->ID, 'braintree_vault_id', true);
	}
	$user = get_current_braintree_user($braintree_id);
	
	$user = apply_filters('get_braintree_user', $user, $gateway);
	?>
	
	<div id="dropin-container"><?php echo $gateway->get_active_environment()?><div class="save-cc"><div><span>Save CC</span></div><input type="checkbox" name="save_cc" value="save"/></div></div>
  <?php echo send_braintree_client_token($user)?>
	<input type="hidden" name="payment_type"/>
	<input type="hidden" id="payment_form" value="dropin"/>
	<input type="hidden" name="payment_method_nonce" id="payment_method_nonce"/>
	  <script>
    jQuery(document).ready(function(){

		 var $ = jQuery;
			var clientToken = $('#client_token').val();
			var amount = $('#order_amount').val();
			var form = $("form[name='checkout']").attr('id', 'checkout');
			
			if(clientToken){
				braintree.setup(clientToken, 'dropin', {container: 'dropin-container',
					form: 'checkout',
					onPaymentMethodReceived: function (response){
						paymentReceived.execute(response);

						}});
			
			}

  })   
  </script>
  <style>
  
  div.save-cc span {
  	font-size: 14px;
  	font-weight: 700;
  }
  div.save-cc input[type="checkbox"] {
  	width: 20px;
  	height: 20px;
  }
  </style>
	<?php 
}

/**
 * This function replaces the standard woocommerce payment-method.php page with an exact replica. This replacement is needed
 * because there is a bug in either woocommerce or braintree that prevents the dropin ui from staying within the html container
 * on the checkout page. 
 * @param unknown $template_name
 * @param unknown $template_path
 * @param unknown $default_path
 */
/* function replace_woocommerce_payment_methods_page($located, $template_name, $args, $template_path, $default_path){
	if($template_name !== 'checkout/payment-method.php') {
		return $located;
	}
	$settings = get_option('woocommerce_braintree_payment_gateway_settings', true);
	if($settings['braintree_dropin_ui'] == 'yes' || $settings['braintree_custom_ui']){
	$located = WC_BRAINTREE_GATEWAY.'includes/templates/checkout/payment-method.php';
		return $located;
	}
	return $located;
}
add_filter('wc_get_template', replace_woocommerce_payment_methods_page, 10, 5); */

function get_hidden_cart_total(){
	$cart = WC()->cart;
	if($cart->prices_include_tax){
		return $cart->cart_contents_total;
	}
	else {
		return ($cart->cart_contents_total + $cart->tax_total);
	}
}
add_filter('get_hidden_cart_total', 'get_hidden_cart_total');

function get_braintree_template_scripts($located, $handle, $script){
	$template_path = get_template_directory();
	
	if(!$template_path){
		return $located;
	}
	if(file_exists($template_path.'/braintree-template/assets/'.$script)){
		return get_template_directory_uri().'/braintree-template/assets/';
	}
	return $located;
}
add_filter('get_braintree_template_scripts', 'get_braintree_template_scripts', 10, 3);

function has_braintree_id($user_id){
	try {
		if($customer = Braintree_Customer::find($user_id)){
			return true;
		}
	}
	catch(Exception $e){
		return false;
	}
}

function get_current_braintree_user($user_id){
	try{
	$user = Braintree_Customer::find($user_id);
		return $user;
	}
	catch(Exception $e){
		return false;
	}
}

function get_paypal_screen($gateway){
	
	if($gateway->paypal_enabled){
		?>
		<div class="paypal-div"><h3><?php echo __('Paypal', 'woocommerce')?></h3></div>
		  					<div id="paypal-container"></div>
		  					<input type="hidden" id="order_amount" value="<?php echo apply_filters('get_hidden_cart_total')?>"/>
		  					<script>
		  					jQuery(document).ready(function(){
		
		  					var $ = jQuery;
		  					var amount = $('#order_amount').val();
		  					var clientToken = $('#client_token').val();			
		
		  					braintree.setup(clientToken, 'paypal', {
		  					  	container: "paypal-container",
		  					  	singleUse: false,
		  					    locale: '<?php echo get_locale()?>',
		  					  	currency: '<?php echo get_woocommerce_currency()?>',
		  					  	editable: false,
		  					  	onPaymentMethodReceived: function (obj) {
		  					  }
		  					});
		
		  					
		  					})
		  					</script>
		  					<?php }
	}
	
	function get_braintree_setup_js($type = 'custom'){
		if($type == 'custom'){
			return '<script type="text/javascript">
 		jQuery(document).ready(function(){

		 var $ = jQuery;
			var clientToken = $("#client_token").val();
			var amount = $("#order_amount").val();
			form = $("form[name=\'checkout\']").attr("id", "checkout");
			
			if(clientToken){
				braintree.setup(clientToken, "custom", {id: "checkout", 
					onPaymentMethodReceived: function (response){
						paymentReceived.execute(response);
							$("#payment_method_nonce").val(response.nonce);
 							$("#payment_type").val(response.type);
						}});
			
			}

  })   
 		</script>';
		}
	}

?>